Package: nlsr
Type: Package
Title: Functions for Nonlinear Least Squares Solutions - Updated 2022
Version: 2026.4.29
Date: 2026-04-29
Authors@R: c(person(given = c("John", "C"), family = "Nash", 
        role = c("aut", "cre"), email = "profjcnash@gmail.com"), 
    	person(given = "Duncan", family = "Murdoch", role = "aut", 
        email = "murdoch.duncan@gmail.com"),
        person(given= "Fernando", family = "Miguez", role="ctb", 
         email="femiguez@iastate.edu"), 
        person(given= "Arkajyoti", family = "Bhattacharjee", role="ctb", 
         email="arkastat98@gmail.com"))
Maintainer: John C Nash <profjcnash@gmail.com>
Description: Provides tools for working with nonlinear least squares problems.
      For the estimation of models reliable and robust tools than nls(), where the
      the Gauss-Newton method frequently stops with 'singular gradient' messages. 
      This is accomplished by using, where possible, analytic derivatives to compute
      the matrix of derivatives and a stabilization of the solution of the estimation
      equations. Tools for approximate or externally supplied derivative matrices
      are included. Bounds and masks on parameters are handled properly.
License: GPL-2
Encoding: UTF-8
Depends: R (>= 3.5)
Imports: digest
Suggests: minpack.lm, optimx, numDeriv, knitr, rmarkdown, markdown,
        Ryacas, Deriv, microbenchmark, MASS, ggplot2, nlraa
VignetteBuilder: knitr
RoxygenNote: 7.2.1
NeedsCompilation: no
Packaged: 2026-04-30 15:33:39 UTC; john
Author: John C Nash [aut, cre],
  Duncan Murdoch [aut],
  Fernando Miguez [ctb],
  Arkajyoti Bhattacharjee [ctb]
Repository: CRAN
Date/Publication: 2026-05-01 11:30:02 UTC
Built: R 4.5.2; ; 2026-05-01 17:05:38 UTC; unix
