S3method(coef,nlsr)
S3method(fitted,nlsr)
S3method(predict,nlsr)
S3method(print,nlsr)
S3method(residuals,nlsr)
S3method(summary,nlsr)
export(SSlogisJN)
export(SSmod2rjfun)
export(codeDeriv)
export(dex)
export(findSubexprs)
export(fnDeriv)
export(isCALL)
export(isMINUSONE)
export(isONE)
export(isZERO)
export(jaback)
export(jacentral)
export(jafwd)
export(jand)
export(model2rjfun)
export(model2ssgrfun)
export(modelexpr)
export(newDeriv)
export(newSimplification)
export(nlfb)
export(nlsDeriv)
export(nlsSimplify)
export(nlsr.control)
export(nlsr)
export(nlsrSS)
export(nlxb)
export(numericDerivR)
export(nvec)
export(pctrl)
export(pnlslm)
export(pnls)
export(prt)
export(pshort)
export(rawres)
export(resgr)
export(resss)
export(sysDerivs)
export(sysSimplifications)
export(wrapnlsr)
importFrom(digest,digest)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,deriv)
importFrom(stats,deviance)
importFrom(stats, getInitial)
importFrom(stats,nls)
importFrom(stats,pt)
importFrom(stats,setNames)
importFrom(utils,str)
