\name{pnls}
\alias{pnls}
\title{pnls}
\usage{
pnls(x)
}
\arguments{
\item{x}{an nls() result object from nls() or nlsLM()}
}
\value{
none
}
\description{
Compact display of specified \code{nls} object \code{x}
}
\author{
J C Nash 2014-7-16, 2023-5-8 nashjc _at_ uottawa.ca
}
