\name{nvec}
\alias{nvec}
\title{nvec}
\description{
Compact display of a specified named vector
}
\usage{
nvec(vec)
}
\arguments{
\item{vec}{a named vector of parameters}
}
\value{
none (Note that we may want to change this.)
}
\author{
J C Nash 2014-7-16   nashjc _at_ uottawa.ca
}
