\name{pshort}
\alias{pshort}
\title{pshort}
\description{
To provide a 1-line summary of an object of class \code{nlsr}.
}
\usage{
pshort(x)
}
\arguments{
\item{x}{an object of class \code{nlsr}}
}
\author{
J C Nash 2022-11-22   nashjc _at_ uottawa.ca
}
