\name{prt}
\alias{prt}
\title{prt}
\description{
To display the calling name of \code{x} and print
the object with the print.nlsr() function.
}
\usage{
prt(x)
}
\arguments{
\item{x}{an object of class \code{nlsr}}
}
\author{
J C Nash 2022-11-22   nashjc _at_ uottawa.ca
}
