\name{pctrl}
\alias{pctrl}
\title{pctrl}
\usage{
pctrl(control)
}
\arguments{
\item{control}{a control list}
}
\value{
none
}
\description{
Compact display of specified \code{control} vector for
package \code{nlsr}.
}
\author{
J C Nash 2014-7-16   nashjc _at_ uottawa.ca
}
